# Main tcl file which includes other tcl modules.
# Rick Smereka, Copyright (C) 2005-2008.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, get a copy via the Internet at
# http://gnu.org/copyleft/gpl.html or write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
# MA 02111-1307 USA

# You can contact the author via email at rsmereka@future-lab.com

# Initial tcl version. Apr/2005, Rick Smereka

# Added requirement of 'TCLDIR' which must point to the tcl source
# base directory. Oct/2005, Rick Smereka

# Added open and close square bracket constants. Jul/2006, Rick Smereka

# Added logic flags as constants. Jul/2008, Rick Smereka

if {![info exists TCLDIR]} {
   puts "stdhead:abort:'TCLDIR' does not exist,it must point to tcl source dir"
   exit
}

# first source the tcl specific module which enables constants
source $TCLDIR/lib/spec.tcl

# define system-wide constants
if {$tcl_platform(platform) == "windows"} {
   const SWITCH_CHAR "/"
}

if {$tcl_platform(platform) == "unix"} {
   const SWITCH_CHAR "-"
}

# logic flags
const TRUE 1
const FALSE 0

# always use '/' as path separator in tcl
const PATH_SEP "/"

# open and close square bracket constants given as ASCII codes since
# this $#&! language freaks-out anytime square brackets as literals
# are used (even in quotes)
const OB 91
const CB 93

# remaining library modules
source $TCLDIR/lib/fio.tcl
source $TCLDIR/lib/logman.tcl
source $TCLDIR/lib/logger.tcl
source $TCLDIR/lib/parse.tcl
source $TCLDIR/lib/fxp.tcl
source $TCLDIR/lib/datime.tcl
source $TCLDIR/lib/misc.tcl

